

// number of handled notes
static int NUM_NOTES = 128;

// note frequences relative to MIDI pitches
static float NOTE_FREQ[] = {
00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, 00.00, // <<< DUMMY!!!!
16.35, 17.32, 18.35, 19.45, 20.60, 21.83, 23.12, 24.50, 25.96, 27.50, 29.14, 30.87,
32.70, 34.65, 36.71, 38.89, 41.20, 43.65, 46.25, 49.00, 51.91, 55.00, 58.27, 61.74,
65.41, 69.30, 73.42, 77.78, 82.41, 87.31, 92.50, 98.00, 103.83, 110.00, 116.54, 123.47,
130.81, 138.59, 146.83, 155.56, 164.81, 174.61, 185.00, 196.00, 207.65, 220.00, 233.08, 246.94,
261.63, 277.18, 293.66, 311.13, 329.63, 349.23, 369.99, 392.00, 415.30, 440.00, 466.16, 493.88,
523.25, 554.37, 587.33, 622.25, 659.25, 698.46, 739.99, 783.99, 830.61, 880.00, 932.33, 987.77,
1046.50, 1108.73, 1174.66, 1244.51, 1318.51, 1396.91, 1479.98, 1567.98, 1661.22, 1760.00, 1864.66, 1975.53,
2093.00, 2217.46, 2349.32, 2489.02, 2637.02, 2793.83, 2959.96, 3135.96, 3322.44, 3520.00, 3729.31, 3951.07,
4186.01, 4434.92, 4698.63, 4978.03, 5274.04, 5587.65, 5919.91, 6271.93, 6644.88, 7040.00, 7458.62, 7902.13 };

// ---- notes stuff

static int NOTE_C  = 0;  // C
static int NOTE_Cs = 1;  // C#
static int NOTE_D  = 2;
static int NOTE_Ds = 3;
static int NOTE_E  = 4;
static int NOTE_F  = 5;
static int NOTE_Fs = 6;
static int NOTE_G  = 7;
static int NOTE_Gs = 8;
static int NOTE_A  = 9;
static int NOTE_As = 10;
static int NOTE_B  = 11;

static int NOTE_Bb  = NOTE_As; // Bb
static int NOTE_Ab  = NOTE_Gs;
static int NOTE_Gb  = NOTE_Fs;
static int NOTE_Eb  = NOTE_Ds;
static int NOTE_Db  = NOTE_Cs;


// ---- MIDI stuff (some octaves )
static int MIDI_C4 = 60;
static int MIDI_D4 = MIDI_C4 + NOTE_D;
static int MIDI_E4 = MIDI_C4 + NOTE_E;
static int MIDI_F4 = MIDI_C4 + NOTE_F;
static int MIDI_G4 = MIDI_C4 + NOTE_G;
static int MIDI_A4 = MIDI_C4 + NOTE_A;
static int MIDI_B4 = MIDI_C4 + NOTE_B;

static int MIDI_C3 = MIDI_C4-12;
static int MIDI_D3 = MIDI_C3 + NOTE_D;
static int MIDI_E3 = MIDI_C3 + NOTE_E;
static int MIDI_F3 = MIDI_C3 + NOTE_F;
static int MIDI_G3 = MIDI_C3 + NOTE_G;
static int MIDI_A3 = MIDI_C3 + NOTE_A;
static int MIDI_B3 = MIDI_C3 + NOTE_B;

static int MIDI_C2 = MIDI_C3-12;
static int MIDI_D2 = MIDI_C2 + NOTE_D;
static int MIDI_E2 = MIDI_C2 + NOTE_E;
static int MIDI_F2 = MIDI_C2 + NOTE_F;
static int MIDI_G2 = MIDI_C2 + NOTE_G;
static int MIDI_A2 = MIDI_C2 + NOTE_A;
static int MIDI_B2 = MIDI_C2 + NOTE_B;

static int MIDI_C1 = MIDI_C2-12;

static int MIDI_C5 = MIDI_C4+12;
static int MIDI_D5 = MIDI_C5 + NOTE_D;
static int MIDI_E5 = MIDI_C5 + NOTE_E;
static int MIDI_F5 = MIDI_C5 + NOTE_F;
static int MIDI_G5 = MIDI_C5 + NOTE_G;
static int MIDI_A5 = MIDI_C5 + NOTE_A;
static int MIDI_B5 = MIDI_C5 + NOTE_B;

static int MIDI_C6 = MIDI_C5+12;
static int MIDI_D6 = MIDI_C6 + NOTE_D;
static int MIDI_E6 = MIDI_C6 + NOTE_E;
static int MIDI_F6 = MIDI_C6 + NOTE_F;
static int MIDI_G6 = MIDI_C6 + NOTE_G;
static int MIDI_A6 = MIDI_C6 + NOTE_A;
static int MIDI_B6 = MIDI_C6 + NOTE_B;

static int MIDI_C7 = MIDI_C6+12;

int fifthCycle[] = { NOTE_C, NOTE_G, NOTE_D, NOTE_A, NOTE_E, NOTE_B, NOTE_Fs, NOTE_Db, NOTE_Ab, NOTE_Eb, NOTE_Bb, NOTE_F };

static String NOTE_LABEL[] = { "C", "C#", "D", "Eb", "E", "F", "F#", "G", "Ab", "A", "Bb", "B" };  

// durations
static int Q8    = 1<<9;
static int Q4    = 1<<8;
static int Q2    = 1<<7;
static int Q1    = 1<<6;
static int Q1_2  = 1<<5;
static int Q1_4  = 1<<4;
static int Q1_8  = 1<<3;
static int Q1_16 = 1<<2;
static int Q1_32 = 1<<1;
static int Q1_64 = 1<<0;

public String noteName( int n ) {
  return NOTE_LABEL[n % 12] + (n/12 - 1);  
}

public String fmtMsec( int msec ) {
   int s = msec / 1000;
   int m = s / 60;
   s = s % 60;
   msec = msec % 1000;
   return "" + m + ":" + nf(s,2) + "." + nf(msec,3); 
}
