/**
 * A video animation for the ugliest music
 * "The Perfect Ping" by Scott Rickard
 * played on a synthesizer.
 *  
 * see https://www.algoritmarte.com/ for further details
 *
 * License: Creative Commons Attribution Share Alike 4.0 
 */
static int NOTES = 88;
static int FPS = 20;
static int BLOCKW = 7;
static int BLOCKH = 7;
int mat[][] = new int[NOTES][NOTES];

int nextnoteframe = 0;
int currnote = 0;
int lastpitch = -1;
boolean eofvideo = false;

boolean fSaveVideo = false;
int numsaved = 0;

int gal[] = { 
  33, 3, 45, 10, 18, 47, 30, 311, 96, 46, 145, 37, 14, 17, 71, 22, 175, 132, 8, 66, 
  41, 251, 11, 50, 67, 59, 39, 23, 56, 34, 241, 2, 133, 35, 157, 204, 6, 108, 29, 
  57, 122, 52, 60, 89, 49, 85, 147, 218, 21, 32, 72, 44, 100, 19, 111, 16, 27, 
  178, 12, 181, 26, 38, 103, 99, 183, 20, 131, 78, 80, 4, 180, 15, 25, 212, 24, 
  286, 54, 9, 92, 5, 159, 7, 87, 42, 172, 13, 69, 1
};

color toRGB(final color h, final color s, final color b) {
  return toRGB(h, s, b, 255);
}

color toRGB(final color h, final color s, final color b, final color a) {
  pushStyle();
  colorMode(HSB, 360, 100, 100, 255);
  final color rgb = color(h, s, b, a);
  popStyle();
  return rgb;
}

Spark sparks[] = new Spark[NOTES];
int ispark = 0;

int xoff = 20;
int yoff = 20;


class Spark {
  boolean enabled = false;
  int x, y;
  int col[] = {0,0,0,0};
  int step = 0;

  Spark() {
    int hue = irand(0, 360);
    int sat = irand(10, 100);
    //int bri = irand(70, 100 - col.length);
    int bri = irand(10, 100 - col.length);
    for (int i = 0; i < col.length; i++) {
      col[i] = toRGB( hue, sat, bri+i, 0xff );
    }
  }

  void draw() {
    boolean fidle = true;
    int dx[] = { 1, 1, -1, -1 };
    int dy[] = { -1, 1, -1, 1 };
    //    int dx[] = { 1,-1,1,-1 };
    //    int dy[] = { -1,-1,1,1 };
    step++;
    for (int z = 0; z < 4; z++ ) {

      for (int i = 0; i <= step; i++) {      
        int x1 = x + dx[z]*i;
        int y1 = y + dy[z]*(step - i);
        if (x1 < 0 || x1 >= NOTES || y1 < 0 || y1 >= NOTES ) {
        } else {
          fidle = false;
          noStroke();
          fill( col[  irand(0,col.length-1)  ] );
          rect( xoff + x1*BLOCKW, yoff + y1*BLOCKH, BLOCKW, BLOCKH );
        }
      }
    }
    if (fidle) enabled = false;
  }
}


void setup() {  
  size(1280, 720, JAVA2D);
  background(0);
  frameRate( FPS );

  int n = 1;
  for (int i = 0; i < NOTES; i++) {
    mat[ i ][n-1] = 1;
    n = ((n * 3)-1) % (NOTES+1) + 1;
    //print("" + n + "," );
  }
  

  for (int i = 0; i < NOTES; i++) {
    sparks[i] = new Spark();
  }

  xoff = (width - BLOCKW*NOTES) / 2;
  yoff = (height - BLOCKH*NOTES) / 2;
  drawMat();
  
    nextnoteframe = FPS*4;
}

void drawMat() {
  strokeWeight(1);
  noStroke();
  for (int y = 0; y < 88; y++ ) {
    for (int x = 0; x < 88; x++) {
      stroke(0xff303030);
      if ( mat[y][x] > 0 ) {
        fill( 0xffffffff );
      } else {
        noFill();
      }
      rect( xoff + x*BLOCKW, yoff + y*BLOCKW, BLOCKW, BLOCKH );
    }
  }
  noFill();
  strokeWeight(4);
  stroke( 0xffffffff);
  rect( xoff-6, yoff-6, NOTES*BLOCKW+12, NOTES*BLOCKH+12);
  strokeWeight(1);
  

}

void draw() {
  
  if ( frameCount >= nextnoteframe ) {
    for (int i = 0; i < NOTES; i++) {
      if ( mat[currnote][i] > 0 ) {
          lastpitch = 21 + i;
          
          break;
      }
    }
    // --------------------
    // if you have a MIDI device you could
    // insert the code here to play the note lastpitch
    // ....
    // --------------------
    nextnoteframe += gal[ currnote ] * 2;
    println( nextnoteframe );
    currnote = (currnote + 1) % NOTES;
  }
  
  if ( lastpitch >= 0 && ! eofvideo) {
    int z = lastpitch-21;
    String s = noteName( lastpitch );
    noStroke();
    fill( 0xff000000);
    rect(  BLOCKW*88 + xoff + 30, 0, width, height );
    fill( 0xffffffff);
    textSize( 30 );
    text( "Note:", BLOCKW*88 + xoff + 60, BLOCKH*44 + yoff - 40);
    text( s, BLOCKW*88 + xoff + 60, BLOCKH*44 + yoff );

    lastpitch = -1;
    for (int i = 0; i < NOTES; i++) {
      if ( mat[i][z] > 0 ) {
        Spark sp = sparks[ispark];
        sp.y = i;
        sp.x = z;
        sp.enabled = true;
        sp.step = 0;
        ispark = ispark + 1;
        if ( ispark == NOTES ) {
          eofvideo = true;
        }
      }
    }
  }

  if ( (frameCount % 4)==0) {
    for (int i = 0; i < NOTES; i++) {
      if ( sparks[i].enabled ) {
        sparks[i].draw();
      }
    }
  }
  drawMat();
  
  int tf = FPS * (60 * 8 + 10);
  if ( fSaveVideo ) {
    saveFrame(  "C:/tmp/video/vid" + nf(numsaved, 5) + ".tga" ); 
    numsaved++;  
    if ( numsaved == tf ) {
      println( "stop video" );
    }
  }  
}
